/*************************************************************************
Title:    SMS Alarm-notifier
Author:   Roland Praml  <praml.roland@t-online.de>  
Date:     March 2003
Software: AVR-GCC with GNU binutiils
          LCD Routines from Peter Fleury
Hardware: HD44780 2x16 compatible LCD text display
          ATS90S8515
**************************************************************************/

#include <io.h>
#include "lcd.h"
#include "gsm.h"
#include <interrupt.h>
#include <sig-avr.h>
#include <eeprom.h>

/*
** constant definitions
*/

#define NONE 0
#define LEFT 1
#define RIGHT 2
#define PRESS 3

/* Display maininfo and alert messages */
#define SIG 0
#define BATT 1

#define SIG_LEVEL 2
#define BATT_LEVEL 3

// keys
#define NONE 0
#define LEFT 1
#define RIGHT 2
#define PRESS 3

// LCD-Pages
#define MAIN 0
#define BLOCK1 1
#define BLOCK2 2
#define BLOCK3 3
#define EDIT 10


char __attribute__ ((section (".eeprom"))) dummy = 'X'; // Do not access 1st byte
char __attribute__ ((section (".eeprom"))) Numbers[21][16] = {
  "888801          ", // 20 Tel-Numbers
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "                ",
  "Einsatzmeldung  "}; // SMS-Text



u08 page =0;
u08 cursor;
u08 editpage;
u08 editing=0;

u16 counter;
static volatile u08 sfreq = 250;

void display (u08 event);

u08 sendsms = 0;
u08 volatile alarm = 0;


void toString(u08 i) {
  SCRATCH[0]=(i / 10) % 10 + '0';
  SCRATCH[1]=i  % 10 + '0';
  SCRATCH[2]=0;
}

void sound(u08 freq) {
  if (freq) {
    sfreq = freq;
    //outp(0x04,TCCR1B); // Freq fr Speaker
    sbi(TCCR1A, COM1A0);  
  } else {
    //outp(0x00,TCCR1B); // Freq fr Speaker
    cbi(TCCR1A, COM1A0);  
    cbi(PORTD, PD5); // Speaker;
  }
}

SIGNAL(SIG_OVERFLOW1)
{
  outp(255, TCNT1H);
  outp(sfreq, TCNT1L);
  counter++;

  if (alarm) {
    switch (alarm) {
    case 1:
      if ((counter & 4095) == 100) sound(0);
      if ((counter & 4095) == 0) sound(245);
      break;
    case 2:
      if ((counter & 511) == 200) sound(0);
      if ((counter & 511) == 0) sound(245);
      break;
    case 3:
      if ((counter & 255) == 0) sound(240);
      if ((counter & 255) == 127) sound(245);
      break;
    }
  } else {
    cbi(TCCR1A, COM1A0);  
    cbi(PORTD, PD5); // Speaker;
  }

}




void display (u08 event) {
  if (page == MAIN && ( event==RIGHT)) {
    page = BLOCK1;
    cursor=0;
    return;
  }

  if (page == BLOCK1 || page == BLOCK2 || page == BLOCK3) {
    if (event == LEFT) {
      if (cursor > 0) {
	cursor--;
	if (cursor <= 15) page=BLOCK2;
	if (cursor <= 7) page=BLOCK1;
      } else page=MAIN;
    }

    if (event == RIGHT) {
      if (cursor < 20) cursor++;
      if (cursor > 7) page=BLOCK2;
      if (cursor > 15) page=BLOCK3;
    }

    if (event == PRESS) {
      editpage=cursor;
      cursor=0;
      page=EDIT;
      editing = 0;
    }
    return;
  } // end Block

  if (page == EDIT) {

    if (!editing) {
      if (event == LEFT) {
	if (cursor > 0) {
	  cursor--;
	} else {
	  // Wert in eeprom speichern
	  for (cursor =0; cursor < 16; cursor++) {
	    eeprom_wb((int)&Numbers[editpage][cursor],SCRATCH[cursor]);
	  }
	  cursor=editpage;
	  page = BLOCK1 + cursor / 8;
	}
      }
      if (event == RIGHT) if (cursor < 15) cursor++;
      if (event == PRESS) editing=1;
    } else { // editing
      if (event == LEFT && SCRATCH[cursor] > ' ') SCRATCH[cursor]--; 
      if (event == RIGHT && ( 
			     (SCRATCH[cursor] < '9') 
			     || ( (SCRATCH[cursor] < 'z') && (editpage == 20) )
			     )) SCRATCH[cursor]++; 
      if (event == PRESS) {
	editing=0;
	if (cursor < 15) cursor++;
      }
      if (SCRATCH[cursor] == '/') SCRATCH[cursor] = ' ';
      if (SCRATCH[cursor] == '!') SCRATCH[cursor] = '0';
      if (SCRATCH[cursor] == ':') SCRATCH[cursor] = 'A';
      if (SCRATCH[cursor] == '@') SCRATCH[cursor] = '9';
    }
  }
}



SIGNAL(SIG_INTERRUPT1) {
  if (bit_is_clear(PIND,PD3)) {
    asm("nop; nop; nop; nop");
    if (bit_is_clear(PINC,PC0)) {
      display(RIGHT);
    } else if (bit_is_clear(PINC,PC1)) {
      display(LEFT);
    } else {
      display(PRESS);
    }
  }
}



/*************************************
 *  Main Info screen
 ************************************/
u08 dispcnt;
void maininfo(void) {
  u08 i, j;

  if (!GSM_connected) {
    alarm = 3;
    lcd_clrscr();
    lcd_puts("FEHLER!\nKeine Verbind.");
    return;
  }
  lcd_command(64 + SIG_LEVEL*8); // go to CG-Adress of SIG-Level
  j = 31;
  for (i = 4; i > 0; i--) { 
    if (GSM_signal >= i) {
      lcd_putc(j);
    } else lcd_putc(0);
    lcd_putc(0);
    j = (j << 1) & 31;
  } 
  
  lcd_command(64 + BATT_LEVEL*8); // go to CG-Adress of SIG-Level
  j = 31;
  for (i = 4; i > 0; i--) { 
    if (GSM_batt >= i) {
      lcd_putc(j);
    } else lcd_putc(0);
    lcd_putc(0);
    j = (j >> 1);
  } 
  
  lcd_gotoxy(0,0);
  lcd_putc(SIG_LEVEL);
  dispcnt++;
  if (dispcnt & 32) {
    switch (GSM_mode) {
    case 0:  // 12345678901234
      lcd_puts("  kein Netz   ");
      break;
    case 1:  // 12345678901234
      lcd_puts("  D1-Telekom  ");
      alarm = 0;
      break;
    case 2:  // 12345678901234
      lcd_puts("    Anruf     ");
      alarm = 1;
      break;
    case 3:  // 12345678901234
      lcd_puts(" PIN eingeben ");
      alarm = 2;
      break;
    case 4:  // 12345678901234
      lcd_puts("AUSGESCHALTET ");
      alarm = 3;
      break;
    default:
      lcd_puts("KEINE VERBIND.");
      alarm = 3;
      break;
    }
  } 


  lcd_gotoxy(15,0);
  lcd_putc(BATT_LEVEL); // signal symbol

  lcd_gotoxy(0,1);
  lcd_putc(SIG); // signal symbol


  /* display date */
  toString(GSM_day);
  lcd_puts(SCRATCH);
  lcd_putc('.');
  toString(GSM_month);
  lcd_puts(SCRATCH);
  lcd_putc('.');
  toString(GSM_year - 2000);
  lcd_puts(SCRATCH);
  
  lcd_putc(' ');
  /* display time */
  toString(GSM_hour);
  lcd_puts(SCRATCH);
  lcd_putc(':');
  toString(GSM_minute);
  lcd_puts(SCRATCH);

  lcd_gotoxy(15,1);
  lcd_putc(BATT); // batt symbol
}

void block(int which) {
  unsigned char i;
  int name;
  lcd_gotoxy(0,0);
  switch (which) {
  case BLOCK1:
    lcd_puts(" A B C D E F G H"); // block 1
    lcd_gotoxy(2*cursor,0);
    break;
  case BLOCK2:
    lcd_puts(" I J K L M N O P"); // block 2
    lcd_gotoxy(2*cursor-16,0);
    break;
  case BLOCK3:
    lcd_puts(" Q R S T SMSText"); // block 2
    lcd_gotoxy(2*cursor-32,0);
    break;
  }
 
  lcd_putc('>');
  lcd_gotoxy(0,1);

  name = (int)Numbers[cursor];
  SCRATCH[16]=0;
  for (i=0; i < 16; i++) {
    SCRATCH[i]=eeprom_rb(name++);
    lcd_putc(SCRATCH[i]);
  }

}

void edit(void) {
  lcd_clrscr();
  lcd_gotoxy(cursor,0);
  if (!editing) {
    lcd_putc('v');
  } else {
    lcd_putc('*');
  }
  lcd_gotoxy(0,1);
  lcd_puts(SCRATCH);
}


void lcd_thread(void) {
  switch (page) {
  case  MAIN:maininfo();
    break;
  case BLOCK1:
  case BLOCK2:
  case BLOCK3:
    block(page);
    break;
  case EDIT:
    edit();
    break;
  }
}

void reset() {
	lcd_clrscr();
	lcd_puts("1. Piepser Reset");
	lcd_gotoxy(0,1);
	lcd_puts("2. druecke Taste");
	alarm = 1;
	page=MAIN;
	while(bit_is_set(PIND, PD3));
	lcd_clrscr();
	lcd_puts("Fuehre Reset");
	lcd_gotoxy(0,1);
	lcd_puts("aus...");
	outp(0x0F,WDTCR);
	for (;;);
}
// init all
int main(void)
{
  int i,j,k;
  u08 NUM[17];
  u08 rr = 0; // round robin variable
  cbi(DDRD,PD3); /* Pin PD3 input */
  sbi(PORTD,PD3);/* Pin PD3 pull-up enabled */
  sbi(GIMSK,INT1); /* INT1 (PD3) enabled */
  sbi(MCUCR,ISC11);
  sbi(TCCR1A, COM1A0);  
  outp((1<<TOIE1), TIMSK); /* enable TCNT0 overflow */
  //outp(0x05,TCCR0); // Freq
  outp(0x04,TCCR1B); // Freq fr Speaker
  outp(255, TCNT1H);
  outp(250, TCNT1L);
  cbi(PORTD, PD5); // Speaker;
  sbi(DDRD, PD5); // Speaker;
  sei();


  outp(0x00,DDRC); /* Pin PC0..7 input */
  outp(0xFF,PORTC);
  lcd_init(LCD_DISP_ON);
  lcd_clrscr();
  lcd_puts("Selbsttest");
  alarm = 4;
  sendsms = 0;
  for(j = 0; j < 30; j++) { 
    sound(220+ j);
    for (i = 0; i < 10000; i++) { 
      asm ("nop; ");
    }
    if (bit_is_clear(PINC,PC2)) sendsms=1;
  }
  alarm = 0;  

  if (sendsms) {
    reset();
  }

  /* UART stuff */
  
  GSM_init();


  
  lcd_command(64); // CG-ram address;
  
  // Antenna and Battsymbol
  lcd_puts("\037\025\025\016\004\004\004\100\006\017\011\011\011\011\017\100");

  /* initialize display, cursor off */
  lcd_clrscr();
 
  //sbi(TIMSK,TOIE0); /* enable TCNT0 overflow (LCD-Thread)*/  
  GSM_connected = 0;
  for (;;) {                           /* loop forever */
    if (GSM_connected) {
      if (sendsms && (GSM_mode == 1)) {
	sendsms = 0;
	//cbi(TIMSK,TOIE0); /* stop LCD-Thread*/
	

	for (k = 0; k < 20; k++) {

	  j = (int)Numbers[k];
	  NUM[16]=0;
	  for (i=0; i < 16; i++) {
	    NUM[i]=eeprom_rb(j++);
	    if (NUM[i]==' ') {
	      NUM[i] = 0;
	      break;
	    }
	  }
	  if (NUM[0]) {
	    rr = 2; // 2 Versuche
	    while (rr--) {
	      j = (int)Numbers[20];
	      SCRATCH[16]=0;
	      for (i=0; i < 16; i++) {
		SCRATCH[i]=eeprom_rb(j++);
	      }
	      if (GSM_sms(NUM,SCRATCH)) break;
	    }
	  }
	}
	reset();
	//sbi(TIMSK, TOIE0); // Start LCD-Messages

      } else {
	rr++;
	if (rr > 1) rr = 0;

	switch (rr) {
	case 0:
	  GSM_network_status();
	  break;
	case 1:
	  GSM_clock();
	  break;
	}
	k=0;
	for(j = 0; j < 50; j++) {
	  lcd_thread();
	  for (i = 0; i < 10000; i++) asm ("nop");
	  if(bit_is_clear(PINC,PC2)) k++;
	}
	if (k  > 40) sendsms = 1;
      }
    } else GSM_link();
    lcd_thread();
  }
}
/*

1x drcken:
 1 2 3 4 5 6 7 8
bei drehen unterstreichen, bei ber 8 drehen:
9 10 11 12 13 14


*/
